import threading
from time import time
from typing import Callable

from api_analytics.core import log_request
from django import conf
from django.core.handlers.wsgi import WSGIRequest
from django.http.response import HttpResponse


class Analytics:
    def __init__(self, get_response: Callable[[WSGIRequest], HttpResponse]) -> None:
        self.get_response = get_response
        self.api_key = getattr(conf.settings, "ANALYTICS_API_KEY", None)

    def __call__(self, request: WSGIRequest) -> HttpResponse:
        start = time()
        response = self.get_response(request)

        json = {
            'api_key': self.api_key,
            'hostname': request.get_host(),
            'path': request.path,
            'user_agent': request.headers['user-agent'],
            'method': request.method,
            'status': response.status_code,
            'framework': 9,
            'response_time': int((time() - start) * 1000),
        }

        threading.Thread(target=log_request, args=(json,)).start()
        return response


import threading
from time import time

from api_analytics.core import log_request

from tornado.web import RequestHandler


class Analytics(RequestHandler):
    def __init__(self, app, res, api_key):
        super().__init__(app, res)
        self.api_key = api_key
        self.start = None

    async def prepare(self):
        self.start = time()

    def on_finish(self):
        json = {
            'api_key': self.api_key,
            'hostname': self.request.host,
            'path': self.request.path,
            'user_agent': self.request.headers['user-agent'],
            'method': self.request.method,
            'status': self.get_status(),
            'framework': 11,
            'response_time': int((time() - self.start) * 1000),
        }

        threading.Thread(target=log_request, args=(json,)).start()
        self.start = None



import threading
from time import time
from typing import Callable

from flask import Flask, Request, Response
from flask_http_middleware import BaseHTTPMiddleware, MiddlewareManager

from api_analytics.core import log_request


def add_middleware(app: Flask, api_key: str) -> None:
    app.wsgi_app = MiddlewareManager(app)
    app.wsgi_app.add_middleware(Analytics, api_key=api_key)


class Analytics(BaseHTTPMiddleware):
    def __init__(self, api_key: str):
        super().__init__()
        self.api_key = api_key

    def dispatch(self, request: Request, call_next: Callable[[Request], Response]):
        start = time()
        response = call_next(request)

        json = {
            'api_key': self.api_key,
            'hostname': request.host,
            'path': request.path,
            'user_agent': request.headers['user-agent'],
            'method': request.method,
            'status': response.status_code,
            'framework': 1,
            'response_time': int((time() - start) * 1000),
        }
        threading.Thread(target=log_request, args=(json,)).start()
        return response
